(* Importieren von Wetterdaten im Csv-Format
   Export in Tagesdaten

    2020 J. Rathlev 24222 Schwentinental
      Web:  www.rathlev-home.de
      Mail: kontakt(a)rathlev-home.de

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Juni 2020
   *)

unit ConvertCsvMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, NumberEd, Vcl.StdCtrls, Vcl.ExtCtrls,
  Vcl.Buttons, Indicators, WinApiUtils, SynEdit, SynMemo;

{$I JrGlobal.pas}

const
  ProgName = 'ConvertCsv';
  Vers = ' - Vers. 1.0';

type
  TCsvFormatSettings = record
    QuoteChar,
    ColSeparator,
    DecSeparator   : char;
    ColWidth       : integer;
    end;

  TCols = (cMinT,cTemp,cMaxT,cRain,cPress);

  TCCForm = class(TForm)
    pnTop: TPanel;
    edtDatPath: TEdit;
    btnExpPath: TBitBtn;
    Label1: TLabel;
    pnSettings: TPanel;
    rgDecimal: TRadioGroup;
    gbQuote: TGroupBox;
    rbNoQuote: TRadioButton;
    rbQuote: TRadioButton;
    rbOtherQuote: TRadioButton;
    edQuote: TEdit;
    gbDelim: TGroupBox;
    rbSpace: TRadioButton;
    rbTab: TRadioButton;
    rbSemi: TRadioButton;
    rbComma: TRadioButton;
    rbOtherDel: TRadioButton;
    edDelim: TEdit;
    rbSlash: TRadioButton;
    gbLine: TGroupBox;
    Label4: TLabel;
    reLine: TRangeEdit;
    udLine: TNumUpDown;
    gbColumns: TGroupBox;
    laDate: TLabel;
    laTemp: TLabel;
    laMinT: TLabel;
    laMaxT: TLabel;
    siMinT: TSpotIndicator;
    siTemp: TSpotIndicator;
    siMaxT: TSpotIndicator;
    laPress: TLabel;
    laRain: TLabel;
    siRain: TSpotIndicator;
    siPress: TSpotIndicator;
    edDateTo: TEdit;
    edTemp: TEdit;
    edMinT: TEdit;
    edMaxT: TEdit;
    edRain: TEdit;
    edPress: TEdit;
    reMinT: TRangeEdit;
    udMinT: TNumUpDown;
    reTemp: TRangeEdit;
    udTemp: TNumUpDown;
    reMaxT: TRangeEdit;
    udMaxT: TNumUpDown;
    reRain: TRangeEdit;
    udRain: TNumUpDown;
    rePress: TRangeEdit;
    udPress: TNumUpDown;
    cbMinT: TCheckBox;
    cbTemp: TCheckBox;
    cbMaxT: TCheckBox;
    cbRain: TCheckBox;
    cbPress: TCheckBox;
    paPreview: TPanel;
    laCsvName: TLabel;
    laLines: TLabel;
    OpenDialog: TOpenDialog;
    btnDataFile: TBitBtn;
    edDateFrom: TEdit;
    Label2: TLabel;
    bbConvert: TBitBtn;
    btnInfo: TBitBtn;
    btnClose: TBitBtn;
    mePreview: TSynMemo;
    bbPreview: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure btnDataFileClick(Sender: TObject);
    procedure rbTabClick(Sender: TObject);
    procedure rbSpaceClick(Sender: TObject);
    procedure rbSemiClick(Sender: TObject);
    procedure rbCommaClick(Sender: TObject);
    procedure rbSlashClick(Sender: TObject);
    procedure rbOtherDelClick(Sender: TObject);
    procedure rgDecimalClick(Sender: TObject);
    procedure rbNoQuoteClick(Sender: TObject);
    procedure rbQuoteClick(Sender: TObject);
    procedure rbOtherQuoteClick(Sender: TObject);
    procedure reSettingsChange(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure bbConvertClick(Sender: TObject);
    procedure btnInfoClick(Sender: TObject);
    procedure btnExpPathClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,ProgVersDate,
    IniName,LastFile,
    DatPath,AppPath : string;
    VersInfo        : TFileVersionInfo;
    ImpSettings     : TCsvFormatSettings;
    dmin,dmax       : TDateTime;
    Cols            : array [TCols] of integer;
    function UpdatePreview : boolean;
    procedure LoadCsv (const AFilename : string);
    procedure StartConvert(FromDate,ToDate : TDateTime);
  public
    { Public-Deklarationen }
  end;

var
  CCForm: TCCForm;

implementation

{$R *.dfm}

uses System.IniFiles, System.DateUtils, System.Math, Winapi.ShlObj,
  PathUtils, StringUtils, ExtStreams, WinShell, WinUtils, MsgDialogs,
  WetSub, ShellDirDlg;

function TryStringToDateTime (s : string; var dt : TDateTime) : boolean;
var
  j1,j2,j3 : integer;
  err : boolean;
begin
  err:=false;
  Result:=false;
  s:=Trim(s);
  if length(s)>0 then begin
    if Pos('-',s)>0 then begin // mit Datum
      j1:=ReadNxtInt(s,'-',YearOf(Now),err);
      j2:=ReadNxtInt(s,'-',MonthOf(Now),err);
      j3:=ReadNxtInt(s,' ',DayOf(Now),err);
      try dt:=EncodeDate(j1,j2,j3); except err:=true; end;
      end
    else begin
      j1:=Pos(':',s); j2:=Pos('.',s);
      if ((j1=0) and (j2>0)) or ((j2>0) and (j2<j1)) then begin
        j3:=ReadNxtInt(s,'.',DayOf(Now),err);
        j2:=ReadNxtInt(s,'.',MonthOf(Now),err);
        j1:=ReadNxtInt(s,' ',YearOf(Now),err);
        try dt:=EncodeDate(j1,j2,j3); except err:=true; end;
        end
      else dt:=Date;
      end;
    if not err and (length(s)>0) then begin  // Zeit
      j1:=ReadNxtInt(s,':',HourOf(Now),err);
      j2:=ReadNxtInt(s,':',MinuteOf(Now),err);
      j3:=ReadNxtInt(s,'.',SecondOf(Now),err);
      dt:=dt+j1*OneHour+j2*OneMinute+j3*OneSecond;
      end;
    Result:=not err;
    end
  else dt:=Date;
  end;

const
  IniExt = 'ini';
  DefPath = 'Wetter';

  CfgSekt = 'Config';
  iniPath     = 'DataPath';
  iniLast     = 'LastFile';
  iniDecSep   = 'DecSeparator';
  iniColSep   = 'ColSeparator';
  iniQuote    = 'QuoteCharacter';
  iniMinT     = 'MinTempCol';
  iniTemp     = 'TempCol';
  iniMaxT     = 'MaxTempCol';
  iniRain     = 'RainCol';
  iniPress    = 'PressCol';

  DefColSep = Semicolon;
  DefQuote = Quote;

procedure TCCForm.FormCreate(Sender: TObject);
var
  IniFile : TIniFile;
begin
  DatPath:=SetDirName(GetDesktopFolder(CSIDL_PERSONAL))+DefPath;
  AppPath:=GetDesktopFolder(CSIDL_APPDATA);
  if length(AppPath)>0 then begin
    AppPath:=SetDirName(AppPath)+AppSubDir;  // Pfad zu Anwendungsdaten
    if not System.SysUtils.ForceDirectories(AppPath) then AppPath:=DatPath;
    end
  else AppPath:=DatPath;
// Versions-Info
  with VersInfo do if GetFileVersion (Application.ExeName,VersInfo) then begin
    // Versionsnr. ohne "Compilierung"
    Version:=DelExt(Version);
    Comments:=' (Vers. '+Version+')';
    end
  else begin
    InternalName:=ProgName; Version:=Vers; Copyright:=CopRgt;
    end;
  with VersInfo do ProgVersName:=InternalName+Comments;
  ProgVersDate:=DateToStr(FileDateToDateTime(FileAge(Application.ExeName)));
  IniName:=Erweiter(AppPath,ExtractFileName(DelExt(Application.ExeName)),IniExt);
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    DatPath:=ReadString(CfgSekt,IniPath,DatPath);
    LastFile:=ReadString(CfgSekt,IniLast,'');
    with ImpSettings do begin
      ColSeparator:=chr(ReadInteger(CfgSekt,IniColSep,ord(DefColSep)));
      QuoteChar:=chr(ReadInteger(CfgSekt,IniQuote,ord(DefQuote)));
      DecSeparator:=chr(ReadInteger(CfgSekt,IniDecSep,ord(FormatSettings.DecimalSeparator)));
      ColWidth:=0;
      end;
    Cols[cMinT]:=ReadInteger(CfgSekt,iniMinT,2);
    Cols[cTemp]:=ReadInteger(CfgSekt,iniTemp,3);
    Cols[cMaxT]:=ReadInteger(CfgSekt,iniMaxT,4);
    Cols[cRain]:=ReadInteger(CfgSekt,iniRain,5);
    Cols[cPress]:=ReadInteger(CfgSekt,iniPress,6);
    Free;
    end;
  ForceDirectories(DatPath);
  end;

procedure TCCForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  with TIniFile.Create(IniName) do begin
    WriteString(CfgSekt,IniPath,DatPath);
    WriteString(CfgSekt,IniLast,LastFile);
    with ImpSettings do begin
      WriteInteger(CfgSekt,IniColSep,ord(ColSeparator));
      WriteInteger(CfgSekt,IniQuote,ord(QuoteChar));
      WriteInteger(CfgSekt,IniDecSep,ord(DecSeparator));
      end;
    WriteInteger(CfgSekt,iniMinT,Cols[cMinT]);
    WriteInteger(CfgSekt,iniTemp,Cols[cTemp]);
    WriteInteger(CfgSekt,iniMaxT,Cols[cMaxT]);
    WriteInteger(CfgSekt,iniRain,Cols[cRain]);
    WriteInteger(CfgSekt,iniPress,Cols[cPress]);
    Free;
    end;
  end;

procedure TCCForm.FormShow(Sender: TObject);
begin
  edtDatPath.Text:=DatPath;
  with ImpSettings do begin
    edDelim.Hide;
    case ColSeparator of
    Space     : rbSpace.Checked:=true;
    Tab       : rbTab.Checked:=true;
    ';'       : rbSemi.Checked:=true;
    ','       : rbComma.Checked:=true;
    '/'       : rbSlash.Checked:=true;
    else with edDelim do begin
        Text:=ColSeparator;
        rbOtherDel.Checked:=true;
        end;
      end;
    with rgDecimal do case DecSeparator of
      Period : ItemIndex:=0;
      ','    : ItemIndex:=1;
      end;
    edQuote.Hide;
    case QuoteChar of
    '"' : rbQuote.Checked:=true;
    #0  : rbNoQuote.Checked:=true;
    else with edQuote do begin
        rbOtherQuote.Checked:=true;
        Text:=QuoteChar; Show;
        end;
      end;
     end;
  cbMinT.Checked:=Cols[cMinT]<>0; reMinT.Value:=Cols[cMinT];
  siMinT.Indicate:=not cbMinT.Checked;
  cbTemp.Checked:=Cols[cTemp]<>0; reTemp.Value:=Cols[cTemp];
  siTemp.Indicate:=not cbTemp.Checked;
  cbMaxT.Checked:=Cols[cMaxT]<>0; reMaxT.Value:=Cols[cMaxT];
  siMaxT.Indicate:=not cbMaxT.Checked;
  cbRain.Checked:=Cols[cRain]<>0; reRain.Value:=Cols[cRain];
  siRain.Indicate:=not cbRain.Checked;
  cbPress.Checked:=Cols[cPress]<>0; rePress.Value:=Cols[cPress];
  siPress.Indicate:=not cbPress.Checked;
  bbConvert.Enabled:=false;
  end;

function TCCForm.UpdatePreview : boolean;
var
  n,i,j  : integer;
  s,t,sd : string;
  dt     : TDateTime;

  function CheckDateTime (s : String) : boolean;
  var
    dt : TDateTime;
  begin
    Result:=TryStringToDateTime(s,dt);
    end;

  function CheckFloat (s : String) : boolean;
  var
//    i,ic : integer;
    x    : double;
    fs   : TFormatSettings;
  begin
    fs:=FormatSettings;
    fs.DecimalSeparator:=ImpSettings.DecSeparator;
    Result:=TryStrToFloat(s,x,fs);
//    s:=RemSp(ReplChars(s,ImpSettings.DecSeparator,Period));
//    if length(s)=0 then Result:=false
//    else begin
//      i:=1;
//      while (i<=length(s)) and CharInSet(s[i],FloatChars) do inc(i);
//      if (i<length(s)) and CharInSet(s[i+1],FloatChars) then Result:=false
//      else begin
//        val(copy(s,1,pred(i)),x,ic);
//        Result:=ic=0;
//        end;
//      end;
    end;

begin
  n:=reLine.Value;
  with mePreView.Lines do if n>Count then n:=Count;
  s:=mePreview.Lines[n-1];
  edMinT.Text:=''; edTemp.Text:=''; edMaxT.Text:='';
  edRain.Text:=''; edPress.Text:='';
  j:=0; sd:=''; Result:=true;
  edDateFrom.Text:=''; edDateTo.Text:='';
  if mePreview.Lines.Count=0 then Exit;
  repeat
    with ImpSettings do if QuoteChar=#0 then t:=ReadNxtStr(s,ColSeparator)
    else t:=ReadNxtQuotedStr(s,ColSeparator,QuoteChar);
    inc(j);
    if length(t)>0 then begin
      if j=1 then sd:=t;      // Datum
      if j=reMinT.Value then edMinT.Text:=t;
      if j=reTemp.Value then edTemp.Text:=t;
      if j=reMaxT.Value then edMaxT.Text:=t;
      if j=reRain.Value then edRain.Text:=t;
      if j=rePress.Value then edPress.Text:=t;
      end;
    until length(s)=0;
  Result:=CheckDateTime(sd);   // kein Datum
  if Result then begin
    cbMinT.Checked:=CheckFloat(edMinT.Text);
    siMinT.Indicate:=not cbMinT.Checked;
    cbTemp.Checked:=CheckFloat(edTemp.Text);
    siTemp.Indicate:=not cbTemp.Checked;
    cbMaxT.Checked:=CheckFloat(edMaxT.Text);
    siMaxT.Indicate:=not cbMaxT.Checked;
    cbRain.Checked:=CheckFloat(edRain.Text);
    siRain.Indicate:=not cbRain.Checked;
    cbPress.Checked:=CheckFloat(edPress.Text);
    siPress.Indicate:=not cbPress.Checked;
    with mePreview do begin
      dmax:=EncodeDate(1900,1,1); dmin:=EncodeDate(2050,12,31);
      for i:=n-1 to Lines.Count-1 do begin
        s:=Lines[i];
        if length(s)>0 then begin
          with ImpSettings do if QuoteChar=#0 then t:=ReadNxtStr(s,ColSeparator)
          else t:=ReadNxtQuotedStr(s,ColSeparator,QuoteChar);
          Result:=TryStringToDateTime(t,dt);
          if Result then begin
            if dt<dmin then dmin:=dt;
            if dt>dmax then dmax:=dt;
            end
          else Break;
          end;
        end;
      if Result then begin
        edDateFrom.Text:=DateToStr(dmin);
        edDateTo.Text:=DateToStr(dmax);
        end
      else ErrorDialog(BottomRightPos(gbLine),Format('Ungltiges Datum in Zeile %u',[i+1]));
      end;
    if Result then begin
      if cbMinT.Checked then Cols[cMinT]:=reMinT.Value else Cols[cMinT]:=0;
      if cbTemp.Checked then Cols[cTemp]:=reTemp.Value else Cols[cTemp]:=0;
      if cbMaxT.Checked then Cols[cMaxT]:=reMaxT.Value else Cols[cMaxT]:=0;
      if cbRain.Checked then Cols[cRain]:=reRain.Value else Cols[cRain]:=0;
      if cbPress.Checked then Cols[cPress]:=rePress.Value else Cols[cPress]:=0;
      end;
    end
  else ErrorDialog(BottomRightPos(gbLine),Format('Ungltiges Datum in Zeile %u',[n]));
  bbConvert.Enabled:=Result;
  end;

procedure TCCForm.LoadCsv (const AFilename : string);
var
  fi      : TFileStream;
begin
  if FileExists(AFilename) then begin
    LastFile:=AFilename;
    fi:=TFileStream.Create(AFilename,fmOpenRead or fmShareDenyRead);
    with mePreview do begin
      Clear;
      Lines.LoadFromStream(fi);
      laCsvName.Caption:='Importdatei: '+LastFile+' ('+IntToStr(Lines.Count)+' Zeilen)';;
      with Gutter do begin
        ShowLineNumbers:=Lines.Count>0;
        if ShowLineNumbers then DigitCount:=Trunc(log10(Lines.Count))+1;
        end;
      end;
    fi.Free;
    UpdatePreview;
    end
  else ErrorDialog(BottomRightPos(btnDataFile),'Datei nicht gefunden: '+AFilename);
  end;

procedure TCCForm.rbCommaClick(Sender: TObject);
begin
  ImpSettings.ColSeparator:=Comma;
  edDelim.Hide;
  UpdatePreview;
  end;

procedure TCCForm.rbNoQuoteClick(Sender: TObject);
begin
  ImpSettings.QuoteChar:=#0;
  edQuote.Hide;
  UpdatePreview;
  end;

procedure TCCForm.rbOtherDelClick(Sender: TObject);
begin
  with edDelim,ImpSettings do begin
    if length(Text)>0 then ColSeparator:=Text[1] else ColSeparator:=',';
    Text:=ColSeparator;
    Show;
    if Active then SetFocus;
    end;
  UpdatePreview;
  end;

procedure TCCForm.rbOtherQuoteClick(Sender: TObject);
begin
  with edQuote,ImpSettings do begin
    if length(Text)>0 then QuoteChar:=Text[1] else QuoteChar:=Quote;
    Text:=QuoteChar;
    Show;
    end;
  UpdatePreview;
  end;

procedure TCCForm.rbQuoteClick(Sender: TObject);
begin
  ImpSettings.QuoteChar:=Quote;
  edQuote.Hide;
  UpdatePreview;
  end;

procedure TCCForm.rbSemiClick(Sender: TObject);
begin
  ImpSettings.ColSeparator:=Semicolon;
  edDelim.Hide;
  UpdatePreview;
  end;

procedure TCCForm.rbSlashClick(Sender: TObject);
begin
  ImpSettings.ColSeparator:=Slash;
  edDelim.Hide;
  UpdatePreview;
  end;

procedure TCCForm.rbSpaceClick(Sender: TObject);
begin
  ImpSettings.ColSeparator:=Space;
  edDelim.Hide;
  UpdatePreview;
  end;

procedure TCCForm.rbTabClick(Sender: TObject);
begin
  ImpSettings.ColSeparator:=Tab;
  edDelim.Hide;
  UpdatePreview;
  end;

procedure TCCForm.reSettingsChange(Sender: TObject);
begin
  UpdatePreview;
  end;

procedure TCCForm.rgDecimalClick(Sender: TObject);
begin
  with ImpSettings do if rgDecimal.ItemIndex=0 then DecSeparator:=Period
  else DecSeparator:=Comma;
  UpdatePreview;
  end;

procedure TCCForm.btnDataFileClick(Sender: TObject);
begin
  with OpenDialog do begin
    InitialDir:=ExtractFilePath(LastFile);
    FileName:=ExtractFilename(LastFile);
    if Execute then LoadCsv(FileName);
    end;
  end;

procedure TCCForm.btnExpPathClick(Sender: TObject);
begin
  if ShellDirDialog.Execute ('Verzeichnis mit Wetterdaten',false,true,true,'',DatPath) then
    edtDatPath.Text:=DatPath;
  end;

procedure TCCForm.btnInfoClick(Sender: TObject);
begin
  InfoDialog(CursorPos(Point(15,15)),ProgVersName+' - '+ProgVersDate+#13+VersInfo.Copyright+sLineBreak
    +'E-Mail: '+EMailAdr);
  end;

procedure TCCForm.StartConvert(FromDate,ToDate : TDateTime);
var
  ft     : TReadTextFile;
  d,m,
  y,yf,yt  : word;
  line,i,j : integer;
  s,t    : string;
  dt     : TDateTime;
  WdBuf  : array of TDatBufTyp;
  x      : double;
  fs     : TFormatSettings;
begin
  fs:=FormatSettings;
  fs.DecimalSeparator:=ImpSettings.DecSeparator;
  yf:=YearOf(FromDate); yt:=YearOf(ToDate);
  SetLength(WdBuf,yt-yf+1);
  for i:=0 to length(WdBuf)-1 do begin // vorhandene Daten einlesen
    y:=yf+i;
    if not ReadData(AddPath(DatPath,CreateFileName(y)),y,WdBuf[i]) then ClearBuffer(y,WdBuf[i]);
    end;
  line:=0;
  ft:=TReadTextFile.Create(LastFile,fmShareDenyRead);
  with ft do while not Eof do begin
    inc(line);
    s:=ReadString;
    if (length(s)>0) and (line>=reLine.Value) then begin
      j:=1;
      with ImpSettings do if QuoteChar=#0 then t:=ReadNxtStr(s,ColSeparator)
      else t:=ReadNxtQuotedStr(s,ColSeparator,QuoteChar);
      TryStringToDateTime(t,dt);
      DecodeDate(dt,y,m,d); i:=y-yf;
      repeat
        with ImpSettings do if QuoteChar=#0 then t:=ReadNxtStr(s,ColSeparator)
        else t:=ReadNxtQuotedStr(s,ColSeparator,QuoteChar);
        inc(j);
        if length(t)>0 then begin
          if (j=Cols[cMinT]) and TryStrToFloat(t,x,fs) then WdBuf[i][m,d].TMin:=x;
          if (j=Cols[cTemp]) and TryStrToFloat(t,x,fs) then WdBuf[i][m,d].T:=x;
          if (j=Cols[cMaxT]) and TryStrToFloat(t,x,fs) then WdBuf[i][m,d].TMax:=x;
          if (j=Cols[cRain]) and TryStrToFloat(t,x,fs) then WdBuf[i][m,d].Regen:=x;
          if (j=Cols[cPress]) and TryStrToFloat(t,x,fs) then WdBuf[i][m,d].Druck:=x;
          end;
        until length(s)=0;
      end;
    end;
  ft.Free;
  if ConfirmDialog(BottomRightPos(bbConvert),'Genderte Wetterdaten speichern?') then begin
    for i:=0 to length(WdBuf)-1 do begin// Daten schreiben
      y:=yf+i;
      s:=AddPath(DatPath,CreateFileName(y));
      t:=NewExt(s,'bak');
      if FileExists(t) then DeleteFile(t);
      RenameFile(s,t);
      WriteData(s,y,WdBuf[i]);
      end;
    end;
  end;

procedure TCCForm.bbConvertClick(Sender: TObject);
begin
  if ConfirmDialog(BottomRightPos(bbConvert),Format('Daten fr den Zeitraum vom %s bis %s importieren?',
      [DateToStr(dmin),DateToStr(dmax)])) then
    StartConvert(dmin,dmax);
  end;

procedure TCCForm.btnCloseClick(Sender: TObject);
begin
  Close;
  end;

end.
