(* Importieren und korrigieren von Wetterdaten im HeavyWeather-Format 1
   Export in Tagesdaten

    2014-2023 J. Rathlev 24222 Schwentinental
      Web:  www.rathlev-home.de
      Mail: kontakt(a)rathlev-home.de

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1   - April 2014
   Vers. 1.7 - Januar 2023 - korrigierter Export nach WinWet
     siehe: https://www.dwd.de/DE/leistungen/klimadatendeutschland/beschreibung_tagesmonatswerte.html
   *)

unit HwDataMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Samples.Spin, Vcl.Grids,
  Vcl.Buttons, System.IniFiles, HListBox, EditVal, NumberUtils, NumberEd,
  Vcl.ComCtrls, WinApiUtils, Vcl.Menus;

const
  Prog = 'Wetterdaten importieren';
  Vers = '1.7';

  DefDataFile = 'history.dat';
  iniExt = '.ini';
  DatExt = 'dat';
  BakExt = 'bak';
  AltExt = 'alt';
  HdLength = 18;
  bgSelected = clHighlight;
  fgSelected = clYellow;

  FixHour = 0;        // in Deutschland blicher Zeitpunkt fr Tageswerte seit 2001
  RainHour = 6;        // in Deutschland blicher Zeitpunkt fr tgl. Niederschlagsmess.
  FixMin =  50;

  Ws4Id : AnsiString = 'WS4data file '+#$1A+#32+#$1E+#0+#0;

type
  TDataIndex = (diNone,diTempI,diTempO,diHumI,diHumO,diPrs,diRain,diSpeed,diDir,diGust,diChill);
  TAuswahl = set of TDataIndex;

const
  defAuswahl : TAuswahl = [diTempI,diHumI,diTempO,diHumO,diChill,diPrs,diRain,diSpeed,diDir,diGust,diChill];
  ExpAuswahl : TAuswahl = [diTempI,diHumI,diTempO,diHumO,diPrs,diRain];

  VarProps : array [diTempI..diChill] of TVarProp =
    ((Name : 'Temp. I'; VUnit : 'C'; Digits : 5; Decimal : 1; Min : -40.0; Max : 60.0; Delta : 5.0),
     (Name : 'Temp. A'; VUnit : 'C'; Digits : 5; Decimal : 1; Min : -40.0; Max : 60.0;  Delta : 7.0),
     (Name : 'Feuchte I'; VUnit : '%'; Digits : 3; Decimal : 0; Min : 0.0; Max : 100.0;  Delta : 20.0),
     (Name : 'Feuchte A'; VUnit : '%'; Digits : 3; Decimal : 0; Min : 0.0; Max : 100.0;  Delta : 30.0),
     (Name : 'Luftdruck'; VUnit : 'hPa'; Digits : 5; Decimal : 0; Min : 850.0; Max : 1150.0;  Delta : 20.0),
     (Name : 'Regen'; VUnit : 'l/m'; Digits : 5; Decimal : 2; Min : 0.0; Max : 100.0; Delta : 30.0),
     (Name : 'Wind'; VUnit : 'km/h'; Digits : 5; Decimal : 1; Min : 0.0; Max : 150.0; Delta : 40.0),
     (Name : 'Richtung'; VUnit : ''; Digits : 5; Decimal : 0; Min : 0.0; Max : 359.0; Delta : 360.0),
     (Name : 'Ben'; VUnit : 'km/h'; Digits : 5; Decimal : 1; Min : 0.0; Max : 150.0; Delta : 40.0),
     (Name : 'gef. Temp.'; VUnit : 'C'; Digits : 5; Decimal : 1; Min : -40.0; Max : 60.0; Delta : 5.0));

{$I JrGlobal.pas}

type
  TDataRecord = record
    DateTime : TDateTime;
    case integer of
    1 : (TempI,TempA,
         HumI,HumA,
         Prs,Rain,
         WSpeed,WDir,WGust,Chill   : double);
    2 : (Val : array [diTempI..diChill] of double);
    end;
  TDataArray = array of TDataRecord;

  TData4Record = packed record             // 72 bytes = $48
    DT    : longint;                       // 4
    TempI : smallint;                      // 2
    Temp  : array [1..15] of smallint;     // 15*2
    HumI  : byte;                          // 1
    Hum   : array [1..15] of byte;         // 15*1
    Prs,
    Rain,
    WSpeed,
    WDir   : word;                         // 4*2
    UnUsed : array [1..6] of word;         // 6*2
    end;

  THeader = array [1..HdLength] of byte;

  THwDataRecord = record
    DT : TDateTime;
    PrsAbs,PrsRel : single;
    Wind : single;
    WDir : cardinal;
    WMax : single;
    RRel,RAbs : single;
    TIn,TOut : single;
    HumIn,HumOut : single;
    Dummy : cardinal;
    end;

  TfrmHwDataMain = class(TForm)
    sgrData: TStringGrid;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    GroupBox3: TGroupBox;
    edtPath: THistoryCombo;
    edtExpPath: TEdit;
    btnExport: TBitBtn;
    btnClose: TBitBtn;
    Label1: TLabel;
    GroupBox4: TGroupBox;
    Label4: TLabel;
    spdFirst: TSpeedButton;
    spdLast: TSpeedButton;
    cbxMonth: TComboBox;
    udnDay: TUpDown;
    speDay: TRangeEdit;
    speYear: TRangeEdit;
    udnYear: TUpDown;
    udnMonth: TUpDown;
    lblCount: TLabel;
    btnInfo: TBitBtn;
    pmData: TPopupMenu;
    itmEditData: TMenuItem;
    itmAverage: TMenuItem;
    OpenDialog: TOpenDialog;
    btnDataFile: TBitBtn;
    btnExpPath: TBitBtn;
    bbMonExport: TBitBtn;
    SaveDialog: TSaveDialog;
    itmDelete: TMenuItem;
    cbNoRain: TCheckBox;
    btnExpYear: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure spdFirstClick(Sender: TObject);
    procedure spdLastClick(Sender: TObject);
    procedure sgrDataDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure sgrDataClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure cbxMonthClick(Sender: TObject);
    procedure udnDayClick(Sender: TObject; Button: TUDBtnType);
    procedure udnYearClick(Sender: TObject; Button: TUDBtnType);
    procedure udnMonthClick(Sender: TObject; Button: TUDBtnType);
    procedure btnExportClick(Sender: TObject);
    procedure btnExpPathClick(Sender: TObject);
    procedure speDayExit(Sender: TObject);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
    procedure btnInfoClick(Sender: TObject);
    procedure itmAverageClick(Sender: TObject);
    procedure itmEditDataClick(Sender: TObject);
    procedure bbMonExportClick(Sender: TObject);
    procedure itmDeleteClick(Sender: TObject);
    procedure btnDataFileClick(Sender: TObject);
    procedure edtPathCloseUp(Sender: TObject);
    procedure btnExpYearClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,ProgVersDate,
    PrgPath,PrgName,
    IniName,AppPath,
    DatFile,
    WsPath,
    ExpPath   : string;
    VersInfo  : TFileVersionInfo;
    MonData,
    LastData  : TDataArray;
    POfs      : integer;
    Auswahl   : TAuswahl;
    function ConvertFromHw (Data : THwDataRecord) : TDataRecord;
    function SelectDataFile : boolean;
    function LoadDataFile : boolean;
    function LoadMonth (FName : string; AMonat,AJahr : word) : boolean;
    function CompareDataSets (const arg1,arg2) : boolean;
    function ExportMonth (const ExpName : string; AMonth : integer) : boolean;
    procedure ShowHeader;
    procedure ShowData;
    procedure SaveData;
  public
    { Public-Deklarationen }
  end;

var
  frmHwDataMain: TfrmHwDataMain;

implementation

{$R *.DFM}


uses Winapi.ShlObj, ShellDirDlg, System.DateUtils, WinShell, StringUtils,
  WinUtils, MsgDialogs, WetSub, PathUtils, StatWind;

{-----------------------------------------------------------------------}
// Hilfsroutinen zum Laden von Daten der Wetterstation (Dat-Format)
function EndOfMonth (DateTime : TDateTime) : TDateTime;
var
  y,m,d : word;
begin
  DecodeDate(DateTime,y,m,d);
  Result:=EncodeDate(y,m,DaysInAMonth(y,m))+1;
  end;

function CheckDate(AJahr,AMonat,DT : integer) : boolean;
var
  y,m,d : word;
begin
  DecodeDate(FileDateToDateTime(DT),y,m,d);
  Result:=(y=AJahr) and (m=AMonat) and (d=DaysInAMonth(y,m));
  end;

{-----------------------------------------------------------------------}
const
  DatSekt = 'Data';
  SelSekt = 'Select';
  ExpSekt = 'Export';

  iniExpPath = 'ExportPath';
  iniWsPath  = 'WsPath';
  iniHwData = 'HwData';
  iniMonth = 'Month';
  iniYear = 'Year';
  iniShow = 'ShowData';
  iniTime = 'SplitTime';
  iniNoRain = 'NoRain';

procedure TfrmHwDataMain.FormCreate(Sender: TObject);
var
  IniFile : TIniFile;
  y,m,d   : word;
  a       : TDataIndex;
  i,j     : integer;
begin
  DecodeDate(Date,y,m,d);
  PrgPath:=ExtractFilePath(Application.ExeName);
  PrgName:=ExtractFileName(DelExt(Application.ExeName));
  DatFile:=SetDirName(GetDesktopFolder(CSIDL_PERSONAL))+DefDataFile;
  AppPath:=GetDesktopFolder(CSIDL_APPDATA);
  if length(AppPath)> 0 then begin
    AppPath:=SetDirName(AppPath)+AppSubDir;  // Pfad zu Anwendungsdaten
    if not ForceDirectories(AppPath) then AppPath:=PrgPath;
    end
  else AppPath:=PrgPath;
// Versions-Info
  with VersInfo do if GetFileVersion (Application.ExeName,VersInfo) then begin
    // Versionsnr. ohne "Compilierung"
    Version:=DelExt(Version);
    Comments:=' (Vers. '+Version+')';
    end
  else begin
    InternalName:=Prog; Version:=Vers; Copyright:=CopRgt;
    end;
  with VersInfo do ProgVersName:=InternalName+Comments;
  ProgVersDate:=DateToStr(FileDateToDateTime(FileAge(Application.ExeName)));
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    DatFile:=ReadString(DatSekt,iniHwData,DatFile);
    ExpPath:=SetDirName(ReadString(DatSekt,iniExpPath,ExtractFilePath(DatFile)));
    WsPath:=SetDirName(ReadString(DatSekt,iniWsPath,ExtractFilePath(DatFile)));
    cbNoRain.Checked:=ReadBool(DatSekt,iniNoRain,false);
    cbxMonth.ItemIndex:=ReadInteger(DatSekt,iniMonth,m)-1;
    speYear.Value:=ReadInteger(DatSekt,iniYear,y);
    AusWahl:=[];
    for a:=diTempI to diChill do begin
      if ReadBool(SelSekt,iniShow+ZStrInt(ord(a)+1,2),false) then AusWahl:=AusWahl+[a];
      end;
    if AusWahl=[] then Auswahl:=defAusWahl;
    Free;
    end;
  edtPath.Text:=DatFile;
  edtExpPath.Text:=ExpPath;
  speDay.Value:=1;
  MonData:=nil; LastData:=nil;
  end;

procedure TfrmHwDataMain.FormDestroy(Sender: TObject);
var
  IniFile : TIniFile;
  a       : TDataIndex;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    WriteString(DatSekt,iniHwData,DatFile);
    WriteString(DatSekt,iniExpPath,ExpPath);
    WriteString(DatSekt,iniWsPath,WsPath);
    WriteBool(DatSekt,iniNoRain,cbNoRain.Checked);
    WriteInteger(DatSekt,iniMonth,cbxMonth.ItemIndex+1);
    WriteInteger(DatSekt,iniYear,speYear.Value);
    for a:=diTempI to diDir do WriteBool (SelSekt,iniShow+ZStrInt(ord(a)+1,2),a in Auswahl);
    Free;
    end;
  end;

procedure TfrmHwDataMain.FormShow(Sender: TObject);
var
  ok,cancel : boolean;
begin
  Caption:=Prog+' (Vers. '+Vers+')';
  edtPath.LoadFromIni(Ininame,DatSekt);
  udnMonth.Position:=cbxMonth.ItemIndex+1;
//  ShowHeader;
  cancel:=false; ok:=true;
  repeat
    if not ok or not FileExists(DatFile) then cancel:=not SelectDataFile;
    if not cancel then ok:=LoadMonth (DatFile,cbxMonth.ItemIndex+1,speYear.Value);
    until cancel or ok;
  if cancel then Close else ShowData;
  end;

{-----------------------------------------------------------------------}
function TfrmHwDataMain.ConvertFromHw (Data : THwDataRecord) : TDataRecord;
var
  j : integer;
begin
  with Result do begin
    DateTime:=Data.DT;
    TempI:=Data.TIn; TempA:=Data.TOut;
    HumI:=Data.HumIn; HumA:=Data.HumOut;
    Prs:=Data.PrsRel; Rain:=Data.RAbs;
    WSpeed:=Data.Wind*3.6; WDir:=Data.WDir*22.5;
    WGust:=Data.WMax*3.6; Chill:=0;
    end;
  end;

function TfrmHwDataMain.CompareDataSets (const arg1,arg2) : boolean;
begin
  Result:=THwDataRecord(arg1).DT<THwDataRecord(arg2).DT;
  end;

{-----------------------------------------------------------------------}
function TfrmHwDataMain.SelectDataFile : boolean;
begin
  with OpenDialog do begin
    InitialDir:=ExtractFilePath(DatFile);
    FileName:=ExtractFilename(DatFile);
    if Execute then begin
      DatFile:=FileName;
      with edtPath do begin
        Text:=DatFile; AddItem(DatFile);
        end;
      Result:=true;
      end
    else Result:=false;
    end;
  end;

(* Daten laden *)
function TfrmHwDataMain.LoadMonth (FName : string;
                                   AMonat,AJahr : word) : boolean;
var
  fInp     : TFileStream;
  RecCount,
  na,ne,
  i,n      : integer;
  HwData   : array of THwDataRecord;
  dt       : TDateTime;
begin
  Result:=false;
  if not FileExists(FName) then begin
    ErrorDialog('Datei: '+FName+sLineBreak+'nicht gefunden!');
    exit;
    end;
  try
    fInp:=TFileStream.Create(FName,fmOpenRead+fmShareDenyNone);
  except
    on EFOpenError do begin
      ErrorDialog(Prog,'Datei: '+FName+sLineBreak+'kann nicht geffnet werden!');
      exit;
      end;
    end;
  RecCount:=fInp.Size div SizeOf(THwDataRecord);
  SetLength(HwData,RecCount);
  na:=-1; ne:=-1;
  dt:=EncodeDate(AJahr,AMonat,1);  // Monatsanfang
  for i:=0 to RecCount-1 do fInp.Read(HwData[i],SizeOf(THwDataRecord));
  if RecCount>1 then QuickSort(HwData[0],length(HwData),SizeOf(THwDataRecord),CompareDataSets);
  for i:=0 to RecCount-1 do begin
    if (na<0) and (HwData[i].DT>=dt) then na:=i;
    if (ne<0) and (HwData[i].DT>=IncMonth(dt,1)) then ne:=i;
    end;
  fInp.Free;
  Result:=true;
  if (RecCount>0) and (na>=0) and (na<>ne) then begin
    if ne<0 then ne:=High(HwData)-na+1
    else ne:=ne-na;
    SetLength(MonData,ne+1);
    for i:=1 to ne do MonData[i]:=ConvertFromHw(HwData[na+i-1]);
    if na>0 then MonData[0]:=ConvertFromHw(HwData[na-1]) //letzter Tag des Vormonats
    else MonData[0]:=MonData[1];
    lblCount.Caption:=IntToStr(ne)+' Stze';
    if na>=0 then begin // letzter Tag des Vormonats
      dt:=IncDay(dt,-1);
      ne:=-1;
      for i:=0 to RecCount-1 do begin
        if (ne<0) and (HwData[i].DT>=dt) then ne:=i;
        end;
      SetLength(LastData,na-ne+1);
      for i:=1 to na-ne do LastData[i]:=ConvertFromHw(HwData[ne+i-1]);
      end
    else LastData:=nil;
    end
  else begin
//    ErrorDialog(BottomRightPos(spdLast),Format('Keine passenden Daten fr %s %u gefunden!',[cbxMonth.Items[AMonat-1],AJahr]));
//+sLineBreak+'Andere Datei laden?') then SelectDataFile;
    MonData:=nil; LastData:=nil;
    end;
  HwData:=nil;
  end;

{-----------------------------------------------------------------------}
(* Kopfzeile anzeigen*)
procedure TfrmHwDataMain.ShowHeader;
var
  j,w     : integer;
  a       : TDataIndex;
begin
  with sgrData do begin
//    RowCount:=2;
    Cells[0,0]:='';
    Cells[1,0]:='Tag - Zeit';
    j:=2;
    for a:=diTempI to diDir do if a in Auswahl then begin
      cells[j,0]:=VarProps[a].Name;
      objects[j,0]:=pointer(a);
      inc(j);
      end;
    ColCount:=j;
    w:=(Width-61) div (ColCount-1);
    if w<55 then begin
      w:=55; ColWidths[0]:=35;
      end
    else ColWidths[0]:=Width-(ColCount-1)*w-32;
    for j:=1 to ColCount-1 do ColWidths[j]:=w;
    for w:=1 to RowCount-1 do for j:=0 to ColCount-1 do begin
      cells[j,w]:=' '; objects[j,w]:=nil;
      end;
    Invalidate;
    end;
  // ohne diese Zeile gibt es bei ShowData bei manchen Monaten eine
  // Zugriffsverletzung $C0000005  - warum ???
  end;

{-----------------------------------------------------------------------}
(* Daten anzeigen*)
procedure TfrmHwDataMain.ShowData;
var
  i,j   : integer;
  a     : TDataIndex;
  s     : string;
  v     : single;
begin
  ShowHeader;
  with sgrData do begin
    RowCount:=length(MonData);
    for i:=1 to High(MonData) do begin
      cells[0,i]:=IntToStr(i);
      cells[1,i]:=FormatDateTime('dd - hh:nn',MonData[i].DateTime);
      j:=2;
      for a:=diTempI to diDir do if a in Auswahl then with VarProps[a] do begin
        v:=MonData[i].Val[a];
        if (v>=Min) and (v<=Max) then begin
          try s:=FloatToStrF(v,ffFixed,Digits,Decimal); except s:=''; end;
          end
        else s:='';
        cells[j,i]:=s;
        inc(j);
        end;
      end;
//    speDayChange(self);
    end;
  end;

procedure TfrmHwDataMain.btnExpPathClick(Sender: TObject);
begin
  if ShellDirDialog.Execute ('Exportpfad',false,true,false,'',ExpPath) then
    edtExpPath.Text:=ExpPath;
  end;

procedure TfrmHwDataMain.speDayExit(Sender: TObject);
var
  j  : integer;
begin
  j:=1;
  while (j<length(MonData)) and (speDay.Value>DayOfTheMonth(MonData[j].DateTime)) do inc(j);
  if j<length(MonData) then with sgrData do begin
    TopRow:=j; Row:=j;
    end
  else begin
    InfoDialog(BottomRightPos(udnDay),'Keine passenden Daten gefunden!');
    speDay.Value:=1;
    end;
  end;

procedure TfrmHwDataMain.spdFirstClick(Sender: TObject);
begin
  speDay.Value:=1;
  if length(MonData)>0 then with sgrData do begin
    Row:=1; Col:=1;
    end;
  end;

procedure TfrmHwDataMain.spdLastClick(Sender: TObject);
var
  n : integer;
begin
  with speDay do begin
    n:=Value;
    Value:=DaysInAMonth(speYear.Value,cbxMonth.ItemIndex+1);
    end;
  if length(MonData)>0 then with sgrData do begin
    Row:=length(MonData)-1; Col:=1;
    end;
  end;

procedure TfrmHwDataMain.udnDayClick(Sender: TObject; Button: TUDBtnType);
begin
//  with speDay do if Button=btNext then Value:=Value+1 else Value:=Value-1;
  speDayExit(Sender);
  end;

procedure TfrmHwDataMain.udnYearClick(Sender: TObject; Button: TUDBtnType);
begin
  with speYear do if Button=btNext then Value:=Value+1 else Value:=Value-1;
  if LoadMonth (DatFile,cbxMonth.ItemIndex+1,speYear.Value) then ShowData;
  end;

procedure TfrmHwDataMain.udnMonthClick(Sender: TObject; Button: TUDBtnType);
begin
  with cbxMonth do if Button=btNext then begin
    if ItemIndex<11 then ItemIndex:=ItemIndex+1
    else begin
      ItemIndex:=0; speYear.Value:=speYear.Value+1;
      end;
    end
  else begin
    if ItemIndex>0 then ItemIndex:=ItemIndex-1
    else begin
      ItemIndex:=11; speYear.Value:=speYear.Value-1;
      end;
    end;
  cbxMonthClick(Sender);
  spdFirstClick(Sender);
  end;

procedure TfrmHwDataMain.cbxMonthClick(Sender: TObject);
begin
  with udnDay do begin
    Max:=DaysInAMonth(speYear.Value,cbxMonth.ItemIndex+1);
    if Position>Max then Position:=Max;
    speDay.Value:=Position;
    end;
//  udnMonth.Position:=cbxMonth.ItemIndex+1;
  if LoadMonth (DatFile,cbxMonth.ItemIndex+1,speYear.Value) then ShowData;
  end;

procedure TfrmHwDataMain.btnCloseClick(Sender: TObject);
begin
  Close;
  end;

function TfrmHwDataMain.LoadDataFile : boolean;
var
  ok,cancel : boolean;
begin
  ok:=LoadMonth (DatFile,cbxMonth.ItemIndex+1,speYear.Value);
  cancel:=false;
  while not ok and not cancel do begin
    cancel:=not SelectDataFile;
    if not cancel then ok:=LoadMonth (DatFile,cbxMonth.ItemIndex+1,speYear.Value);
    end;
  if ok then ShowData;
  end;

procedure TfrmHwDataMain.btnDataFileClick(Sender: TObject);
begin
  if SelectDataFile then LoadDataFile;
  end;

procedure TfrmHwDataMain.edtPathCloseUp(Sender: TObject);
begin
  with edtPath do DatFile:=Items[ItemIndex];
  LoadDataFile;
  end;

procedure TfrmHwDataMain.sgrDataDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  with sgrData,Canvas do begin
    if (ARow=0) or (gdFixed in State) then begin
      Brush.Color:=clActiveCaption;
      Font.Color:=clWindowText;
      FillRect (Rect);
      with Rect do Textout(Left+2,Top+2,cells[ACol,ARow]);
      end
    else if (gdSelected in State) and (Col>1) then begin
      Brush.Color:=bgSelected;
      Font.Color:=fgSelected;
      FillRect (Rect);
      with Rect do Textout(Left+2,Top+2,cells[ACol,ARow]);
      end
    else begin
      Brush.Color:=clWindow;
      Font.Color:=clWindowText;
      FillRect (Rect);
      with Rect do Textout(Left+2,Top+2,cells[ACol,ARow]);
      end;
    end;
  end;

procedure TfrmHwDataMain.sgrDataClick(Sender: TObject);
begin
  speDay.Value:=DayOfTheMonth(MonData[sgrData.Row].DateTime);
  sgrData.Invalidate;
  end;

procedure TfrmHwDataMain.itmAverageClick(Sender: TObject);
var
  n  : integer;
  xm : double;
  a  : TDataIndex;
begin
  with sgrData do a:=TDataIndex(Objects[Col,0]);
  if (a>=diTempI) and (a<=diDir) then begin
    xm:=0;
    for n:=1 to High(MonData) do xm:=xm+MonData[n].Val[a];
    if a=diRain then begin
      with VarProps[a] do InfoDialog(CursorPos,Prog,'Summe '+Name+': '+
          FloatToStrF(xm,ffFixed,Digits,Decimal)+' '+VUnit);
      end
    else begin
      with VarProps[a] do InfoDialog(CursorPos,Prog,'Mittelwert '+Name+': '+
          FloatToStrF(xm/length(MonData),ffFixed,Digits,Decimal)+' '+VUnit);
      end;
    end;
  end;

procedure TfrmHwDataMain.itmDeleteClick(Sender: TObject);
var
  i,n1,n2 : integer;
  a       : TDataIndex;
begin
  with sgrData do if (Col>1) and (Row>0) then begin
    with Selection do begin
      if Left<>Right then begin
        ErrorDialog('Es knnen nur mehrere Felder in einer Spalte ausgewhlt werden!');
        exit;
        end;
      n1:=Top; n2:=Bottom;
      end;
    a:=TDataIndex(Objects[Col,0]);
    for i:=n1 to n2 do if (i>0) and (i<=High(MonData)) then
      MonData[i].Val[a]:=NoVal;
    ShowData;
    end;
  end;

procedure TfrmHwDataMain.itmEditDataClick(Sender: TObject);
var
  i,n1,n2 : integer;
  a       : TDataIndex;
  xl,xn   : double;
begin
  with sgrData do if (Col>1) and (Row>0) then begin
    with Selection do begin
      if Left<>Right then begin
        ErrorDialog('Es knnen nur mehrere Felder in einer Spalte ausgewhlt werden!');
        exit;
        end;
      n1:=Top; n2:=Bottom;
      end;
    a:=TDataIndex(Objects[Col,0]);
    xl:=NoVal; xn:=NoVal;
    if n1>0 then xl:=MonData[n1-1].Val[a];
    if n2<length(MonData)-1 then xn:=MonData[n2+1].Val[a];
    if EditValDialog.Execute (CursorPos,VarProps[a],xl,xn,MonData[n1].Val[a]) then begin
      for i:=n1 to n2 do begin
        if i>n1 then MonData[i].Val[a]:=MonData[n1].Val[a];
        with VarProps[a] do cells[Col,i]:=FloatToStrF(MonData[n1].Val[a],ffFixed,Digits,Decimal);
        end;
      end;
    end;
  end;

// Daten sichern
procedure TfrmHwDataMain.SaveData;
var
  fOut     : TFileStream;
  i        : integer;
  Data4    : TData4Record;
  sd,sb    : string;
  ok       : boolean;

  function CreateWsName (Monat,Jahr : word) : string;
  begin
    Result:='WS_'+ZStrInt(MOnat,2)+'_'+ZStrInt(Jahr mod 100,2);
    end;

  function ConvertToWs4 (Data : TDataRecord) : TData4Record;
  var
    j : integer;
  begin
    with Result do begin
      DT:=DateTimeToFileDate(Data.DateTime);
      TempI:=round(10*Data.TempI);
      Temp[1]:=round(10*Data.TempA);
      for j:=2 to 15 do Temp[j]:=0;
      HumI:=round(Data.HumI);
      Hum[1]:=round(Data.HumA);
      for j:=2 to 15 do Hum[j]:=0;
      Prs:=round(Data.Prs); Rain:=round(1000*Data.Rain);
      WSpeed:=round(10*Data.WSpeed); WDir:=round(Data.WDir+0.1);
      end;
    end;

begin
  sd:=Erweiter(WsPath,CreateWsName(cbxMonth.ItemIndex+1,speYear.Value),DatExt);
  with SaveDialog do begin
    DefaultExt:=Datext;
    Filter:='WS-Daten|*'+DatExt;
    InitialDir:=WsPath;
    FileName:=sd;
    ok:=Execute;
    if ok then begin
      sd:=FileName;
      WsPath:=ExtractFilePath(FileName);
      end;
    end;
  if ok then begin
    sb:=NewExt(sd,BakExt);
    ok:=true;
    if FileExists(sb) then begin
      if DeleteFile(sb) then RenameFile(sd,sb)
      else ok:=ConfirmDialog(TopRightPos(bbMonExport),Prog,'Die Sicherheitskopie kann nicht angelegt werden!'+sLineBreak+
                       'Sollen die Daten trotzdem gespeichert werden?');
      end
    else RenameFile(sd,sb);
    end;
  if ok then begin
    fOut:=TFileStream.Create(sd,fmCreate);
    with fOut do begin
      Write(Ws4ID[1],HdLength);
      for i:=1 to High(MonData) do begin
        Data4:=ConvertToWs4(MonData[i]);
        Write(Data4,SizeOf(TData4Record));
        end;
      Free;
      end;
    end;
  end;

procedure TfrmHwDataMain.bbMonExportClick(Sender: TObject);
begin
  SaveData;
  end;

procedure TfrmHwDataMain.btnExportClick(Sender: TObject);
var
  ExpName    : string;
begin
  Expname:=Erweiter(ExpPath,CreateFileName(speYear.Value),'');
  if ExportMonth(Expname,cbxMonth.ItemIndex+1) then
    InfoDialog(TopRightPos(btnExport),Prog,'Daten fr '+cbxMonth.Text+' '+speYear.Text+' wurden nach'#13+
             ExpName+' gespeichert!')
  else ErrorDialog('Export nach WinWet','Zu wenig Datenstze vorhanden!');
  end;

procedure TfrmHwDataMain.btnExpYearClick(Sender: TObject);
var
  i,n : integer;
  ExpName    : string;
  sw         : TStatusWindow;
begin
  if ConfirmDialog(TopRightPos(btnExpYear),Format('Das ganze Jahr %u exportieren?',[speYear.Value])) then begin
    sw:=CreateStatusWindow(TopRightPos(btnExpYear),Prog,'Exportiere Daten nach WinWet',false,0);
    Expname:=Erweiter(ExpPath,CreateFileName(speYear.Value),'');
    n:=0;
    for i:=1 to 12 do begin
      if LoadMonth (DatFile,i,speYear.Value) then begin
        if ExportMonth(Expname,i) then begin
          sw.Info:='Monat: '+cbxMonth.Items[i-1]+' '+speYear.Text;
          Application.ProcessMessages;
          inc(n);
          end;
        end;
      end;
    ReleaseStatusWindow(sw);
    if n=0 then InfoDialog(TopRightPos(btnExpYear),Prog,'Keine Daten zum Exportieren gefunden!')
    else InfoDialog(TopRightPos(btnExpYear),Prog,Format('Daten fr %u Monate im Jahr %u wurden nach'+sLineBreak+
             ExpName+' gespeichert!',[n,speYear.Value]));
    LoadMonth (DatFile,cbxMonth.ItemIndex+1,speYear.Value);
    end;
  end;

function TfrmHwDataMain.ExportMonth (const ExpName : string; AMonth : integer) : boolean;
var
  ExpBuffer  : TDatBufTyp;
  Zt         : TDateTime;
  kt,kd,
  i,m,n,d    : integer;
  NData      : TDataRecord;
  ende,rende : Boolean;
  v          : single;
begin
  if not ReadData (ExpName,speYear.Value,ExpBuffer) then
    ClearBuffer (speYear.Value,ExpBuffer);
  m:=1; ende:=false;
  Zt:=EncodeTime(FixHour,FixMin,0,0);  // Anfangsuhrzeit
  while (m<=length(MonData)) and (frac(MonData[m].DateTime)>Zt) do inc(m);  // bis 0:0
  while (m<=length(MonData)) and (frac(MonData[m].DateTime)<Zt) do inc(m);  // ab 0:0 bis Vorgabezeit
  if m<=length(MonData) then begin
    d:=0; n:=0;
    Zt:=Zt+EncodeDate(speYear.Value,AMonth,1);  // Zeitpunkt fr den ersten Datenwert
    repeat
      inc(d); kt:=0; kd:=0;
      if d<=DaysInAMonth(speYear.Value,AMonth) then with ExpBuffer[AMonth,d] do begin
        TMin:=99.0; T:=0; TMax:=-99.0; Druck:=0;
        repeat
          NData:=MonData[m];
          ende:=(m=length(MonData)) or (NData.DateTime>=d+Zt);
          if not ende then begin
            with NData do begin
              v:=Val[diTempO];
              with VarProps[diTempO] do if (v>=Min) and (v<=Max) then begin
                if TMin>v then TMin:=v;
                if TMax<v then TMax:=v;
                T:=T+v;
                inc(kt);
                end;
              v:=Val[diPrs];
              with VarProps[diPrs] do if (v>=Min) and (v<=Max) then begin
                Druck:=Druck+v;
                inc(kd);
                end;
              end;
            inc(m);
            end;
          until ende;
        if kt>0 then T:=T/kt
        else begin
          TMax:=TNoVal; T:=TNoVal; TMax:=TNoVal;
          end;
        if kd>0 then Druck:=Druck/kd+POfs/8.5
        else Druck:=DNoVal;
        ende:=(m=length(MonData));
        end
      else ende:=true;
      until ende;
    end;
  m:=1;
  if not cbNoRain.Checked then begin
    Zt:=EncodeTime(RainHour,FixMin,0,0);  // Anfangsuhrzeit fr Regen
    while (m<=length(MonData)) and (TimeOf(MonData[m].DateTime)>Zt) do inc(m);  // bis 0:0
    while (m<=length(MonData)) and (TimeOf(MonData[m].DateTime)<Zt) do inc(m);  // ab 0:0 bis Vorgabezeit
    if m<=length(MonData) then begin
      d:=0; n:=0;
      if LastData=nil then m:=1
      else begin
        n:=High(LastData); m:=n;
        while (m>0) and ((TimeOf(LastData[m].DateTime)>Zt)) do dec(m);
        m:=m-n;
        end;
      Zt:=Zt+EncodeDate(speYear.Value,AMonth,1)-1;  // Zeitpunkt fr Ende des 0. Tages
      repeat
        inc(d); kt:=0; kd:=0;
        if d<=DaysInAMonth(speYear.Value,AMonth) then with ExpBuffer[AMonth,d] do begin
          Regen:=0;
          repeat
            if m<1 then NData:=LastData[n+m] else NData:=MonData[m];
            rende:=(m=length(MonData)) or (NData.DateTime>=d+Zt);
            if not rende then begin
              with NData do begin
                v:=Val[diRain];
                with VarProps[diRain] do if (v>=Min) and (v<=Max) then Regen:=Regen+v;
                end;
              inc(m);
              end;
            until rende;
          rende:=(m=length(MonData));
          end
        else rende:=true;
        until rende;
      end;
    end;
  Result:=ende;
  if ende then WriteData (ExpName,speYear.Value,ExpBuffer);
  end;

procedure TfrmHwDataMain.EditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then begin
    Key:= #0;
     { Shift Taste fr vorwrts - rckwrts abfragen }
    if GetKeyState(VK_Shift) and $8000 <> 0 then PostMessage(Handle,WM_NextDlgCtl,1,0)
    else PostMessage(Handle,WM_NextDlgCtl,0,0);
    end;
  end;

procedure TfrmHwDataMain.btnInfoClick(Sender: TObject);
begin
  InfoDialog(Prog,ProgVersName+' - '+ProgVersDate+#13+VersInfo.Copyright+#13+'E-Mail: '+EMailAdr);
  end;

end.
