(* Wetterdaten: Anzeige der Minima und Maxima

    2007-2023 J. Rathlev 24222 Schwentinental
      Web:  www.rathlev-home.de
      Mail: kontakt(a)rathlev-home.de

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   letzte nderung: Juli 2023
   *)

unit MinMaxDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.ComCtrls,
  Vcl.Samples.Spin, NumberEd;

type
  TMValue = record
    Wert  : double;
    Tag,Monat,Jahr : word;
    end;

  TMinMaxDialog = class(TForm)
    OKBtn: TBitBtn;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    GroupBox2: TGroupBox;
    Label10: TLabel;
    leTagMinT: TLabeledEdit;
    leTagMaxT: TLabeledEdit;
    leTagMinP: TLabeledEdit;
    leTagN: TLabeledEdit;
    Label3: TLabel;
    leTagMaxP: TLabeledEdit;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    cbVonMonat: TComboBox;
    seVonJahr: TSpinEdit;
    cbBisMonat: TComboBox;
    seBisJahr: TSpinEdit;
    edTagMinT: TEdit;
    edTagMaxT: TEdit;
    edTagMinP: TEdit;
    edTagMaxP: TEdit;
    edTagN: TEdit;
    GroupBox3: TGroupBox;
    leMonN: TLabeledEdit;
    Label7: TLabel;
    edMonN: TEdit;
    GroupBox4: TGroupBox;
    Label8: TLabel;
    leJahrN: TLabeledEdit;
    edJahrN: TEdit;
    meHinweis: TMemo;
    sbTagMinT: TSpeedButton;
    sbTagMaxT: TSpeedButton;
    sbTagMinP: TSpeedButton;
    sbMaxTagP: TSpeedButton;
    sbTagN: TSpeedButton;
    sbMonN: TSpeedButton;
    sbJahrN: TSpeedButton;
    GroupBox5: TGroupBox;
    leRain: TLabeledEdit;
    Label9: TLabel;
    btnRefresh: TBitBtn;
    procedure btShowClick(Sender: TObject);
    procedure edTagMinTDblClick(Sender: TObject);
    procedure edTagMaxTDblClick(Sender: TObject);
    procedure edTagMinPDblClick(Sender: TObject);
    procedure edTagMaxPDblClick(Sender: TObject);
    procedure edTagNDblClick(Sender: TObject);
    procedure edMonNDblClick(Sender: TObject);
    procedure edJahrNDblClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure seVonJahrChange(Sender: TObject);
    procedure seBisJahrChange(Sender: TObject);
    procedure cbVonMonatCloseUp(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure cbBisMonatCloseUp(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnRefreshClick(Sender: TObject);
  private
    { Private declarations }
    DPath : string;
    mdiff   : integer;
    dmint,dmaxt,dminp,
    dmaxp,dmaxn,mmaxn,ymaxn : TMValue;
    CanChange               : boolean;
    procedure ComputeData;
    procedure UpdateJahrN;
    procedure ShowMonth (mvalue : TMValue);
  public
    { Public declarations }
    procedure ShowData(APath : string; Monat,Jahr : word);
  end;

var
  MinMaxDialog: TMinMaxDialog;

implementation

{$R *.DFM}

uses WetMain, WetSub, NumberUtils;

procedure TMinMaxDialog.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  HauptForm.MinMaxClosed;
  end;

procedure TMinMaxDialog.FormCreate(Sender: TObject);
begin
  mdiff:=12;
  CanChange:=false;
  end;

procedure TMinMaxDialog.OKBtnClick(Sender: TObject);
begin
  Close;
  HauptForm.MinMaxView.Down:=false;
  end;

procedure TMinMaxDialog.ComputeData;
var
  iy,im,id,n    : integer;
  rm,ry,rd,r    : double;
  yerr          : array of integer;
  buf           : TDatBufTyp;
  fnd           : TDatRecTyp;
  s,sd          : string;
  ok            : boolean;

  procedure Reset (var mv : TMValue; def : double);
  begin
    with mv do begin
      Wert:=def; Tag:=0; Monat:=0; Jahr:=0;
      end;
    end;

  function DatumText(d,m,y : word) : string;
  begin
    if d>0 then Result:=IntToStr(d)+'.' else Result:='';
    if m>0 then Result:=Result+IntToStr(m)+'.';
    if iy>0 then Result:=Result+IntToStr(y) else Result:='';
    end;

begin
  Reset(dmint,100); Reset(dmaxt,-100); Reset(dminp,1500); Reset(dmaxp,500); Reset(dmaxn,0);
  Reset(mmaxn,0); Reset(ymaxn,0);
  SetLength(yerr,0); rd:=0;
  for iy:=seVonJahr.Value to seBisJahr.Value do begin
    sd:=DPath+CreateFileName(iy);
    if ReadData(sd,iy,buf) then begin
      ReadFirstNextDay(DPath,iy,fnd);
      ry:=0;
      for im:=1 to 12 do begin
        if (seVonJahr.Value=seBisJahr.Value) then
          ok:=(im>=cbVonMonat.ItemIndex+1) and (im<=cbBisMonat.ItemIndex+1)
        else
          ok:=((iy=seVonJahr.Value) and (im>=cbVonMonat.ItemIndex+1)) or
            ((iy>seVonJahr.Value) and (iy<seBisJahr.Value)) or
            ((iy=seBisJahr.Value) and (im<=cbBisMonat.ItemIndex+1));
        if ok then begin
          rm:=0;
          for id:=1 to MonatsTage(im,iy) do with buf[im,id] do begin
            with dmint do if Valid(TMin,TNoVal) and (TMin<Wert) then begin
              Wert:=TMin; Tag:=id; Monat:=im; Jahr:=iy;
              end;
            with dmaxt do if Valid(TMax,TNoVal) and (TMax>Wert) then begin
              Wert:=TMax; Tag:=id; Monat:=im; Jahr:=iy;
              end;
            with dminp do if Valid(Druck,DNoVal) and (Druck<Wert) then begin
              Wert:=Druck; Tag:=id; Monat:=im; Jahr:=iy;
              end;
            with dmaxp do if Valid(Druck,DNoVal) and (Druck>Wert) then begin
              Wert:=Druck; tag:=id; Monat:=im; Jahr:=iy;
              end;
            with dmaxn do if Valid(Regen,RNoVal) and (Regen>Wert) then begin
              Wert:=Regen;
              if id>1 then begin
                Tag:=id-1; Monat:=im; Jahr:=iy;
                end
              else if im>1 then begin
                Monat:=im-1; Tag:=MonatsTage(Monat,iy); Jahr:=iy;
                end
              else begin
                Tag:=31; Monat:=12; Jahr:=iy-1;
                end;
              end;
            if (id>1) and Valid(Regen,RNoVal) then begin
              rm:=rm+Regen; ry:=ry+Regen; rd:=rd+Regen;
              end;
            end;
          if im<12 then r:=buf[im+1,1].regen
          else r:=fnd.Regen;
          if not Valid (r,RNoval) then r:=0.0;
          rm:=rm+r; ry:=ry+r; rd:=rd+r;
          with mmaxn do if (rm>Wert) then begin
            Wert:=rm; Monat:=im; Jahr:=iy;
            end;
          end;
        end;
      with ymaxn do if (ry>Wert) then begin
        Wert:=ry; Jahr:=iy;
        end;
      end
    else begin
      n:=length(yerr);
      SetLength(yerr,n+1);
      yerr[n]:=iy;
      end;
    end;
  leRain.Text:=FloatToStrX(rd,5,1);
  with dmint do begin
    leTagMinT.Text:=FloatToStrX(Wert,5,1);
    edTagMinT.Text:=DatumText(Tag,Monat,Jahr);
    end;
  with dmaxt do begin
    leTagMaxT.Text:=FloatToStrX(Wert,5,1);
    edTagMaxT.Text:=DatumText(Tag,Monat,Jahr);
    end;
  with dminp do begin
    leTagMinP.Text:=FloatToStrX(Wert,5,0);
    edTagMinP.Text:=DatumText(Tag,Monat,Jahr);
    end;
  with dmaxp do begin
    leTagMaxP.Text:=FloatToStrX(Wert,5,0);
    edTagMaxP.Text:=DatumText(Tag,Monat,Jahr);
    end;
  with dmaxn do begin
    leTagN.Text:=FloatToStrX(Wert,5,1);
    edTagN.Text:=DatumText(Tag,Monat,Jahr);
    end;
  with mmaxn do begin
    leMonN.Text:=FloatToStrX(Wert,5,1);
    edMonN.Text:=DatumText(Tag,Monat,Jahr);
    end;
  with ymaxn do begin
    leJahrN.Text:=FloatToStrX(Wert,5,1);
    edJahrN.Text:=DatumText(Tag,Monat,Jahr);
    end;
  if length(yerr)>0 then with meHinweis do begin
    Clear; Show;
    Lines.Add('Fehler beim Laden der Daten fr:');
    s:='';
    for iy:=0 to High(yerr) do s:=s+IntToStr(yerr[iy])+',';
    Delete(s,length(s),1);
    Lines.Add(s);
    end
  else meHinweis.Hide;
  end;

procedure TMinMaxDialog.UpdateJahrN;
begin
  with leJahrN.EditLabel do if mdiff>12 then Caption:='max. Niederschlag'
  else Caption:='Niederschlag';
  end;

procedure TMinMaxDialog.ShowMonth (mvalue : TMValue);
begin
  with mvalue do begin
    HauptForm.MonatAnzeigen(Monat,Jahr);
    end;
  end;

procedure TMinMaxDialog.edTagMinTDblClick(Sender: TObject);
begin
  ShowMonth(dmint);
  end;

procedure TMinMaxDialog.edTagMaxTDblClick(Sender: TObject);
begin
  ShowMonth(dmaxt);
  end;

procedure TMinMaxDialog.edTagMinPDblClick(Sender: TObject);
begin
  ShowMonth(dminp);
  end;

procedure TMinMaxDialog.edTagMaxPDblClick(Sender: TObject);
begin
  ShowMonth(dmaxp);
  end;

procedure TMinMaxDialog.edTagNDblClick(Sender: TObject);
begin
  ShowMonth(dmaxn);
  end;

procedure TMinMaxDialog.edMonNDblClick(Sender: TObject);
begin
  ShowMonth(mmaxn);
  end;

procedure TMinMaxDialog.edJahrNDblClick(Sender: TObject);
begin
  ShowMonth(ymaxn);
  end;

procedure TMinMaxDialog.btShowClick(Sender: TObject);
begin
  ComputeData;
  end;

procedure TMinMaxDialog.cbBisMonatCloseUp(Sender: TObject);
var
  dm : integer;
begin
  if CanChange then begin
    dm:=12*seBisJahr.Value+cbBisMonat.ItemIndex-mdiff+1;
    cbVonMonat.ItemIndex:=dm mod 12;
    seVonJahr.Value:=dm div 12;
    ComputeData;
    end;
  end;

procedure TMinMaxDialog.cbVonMonatCloseUp(Sender: TObject);
begin
  if CanChange then begin
    mdiff:=12*(seBisJahr.Value-seVonJahr.Value)+cbBisMonat.ItemIndex-cbVonMonat.ItemIndex+1;
    ComputeData;
    UpdateJahrN;
    end;
  end;

procedure TMinMaxDialog.seBisJahrChange(Sender: TObject);
begin
  if Visible and CanChange then begin
    seVonJahr.Value:=seBisJahr.Value-mdiff div 12;
    ComputeData;
    end;
  end;

procedure TMinMaxDialog.seVonJahrChange(Sender: TObject);
begin
  if Visible and CanChange then begin
    mdiff:=12*(seBisJahr.Value-seVonJahr.Value)+cbBisMonat.ItemIndex-cbVonMonat.ItemIndex+1;
    ComputeData;
    UpdateJahrN;
    end;
  end;

procedure TMinMaxDialog.btnRefreshClick(Sender: TObject);
var
  mstart : integer;
  m,j : word;
begin
  CanChange:=false;
  UpdateJahrN;
  with HauptForm do begin
    m:=Monat; j:=Jahr;
    end;
  mstart:=12*j+m-mdiff;
  cbVonMonat.ItemIndex:=mstart mod 12;
  seVonJahr.Value:=mstart div 12;
  cbBisMonat.ItemIndex:=m-1;
  seBisJahr.Value:=j;
  CanChange:=true;
  ComputeData;
  end;


procedure TMinMaxDialog.ShowData(APath : string; Monat,Jahr : word);
var
  mstart : integer;
begin
//  if mstart=0 then begin
  CanChange:=false;
  UpdateJahrN;
  mstart:=12*Jahr+Monat-mdiff;
  cbVonMonat.ItemIndex:=mstart mod 12;
  seVonJahr.Value:=mstart div 12;
  cbBisMonat.ItemIndex:=Monat-1;
  seBisJahr.Value:=Jahr;
  CanChange:=true;
//    end;
  DPath:=APath;
  ComputeData;
  if not Visible then Show;
//  if Result then begin     // nach Doppelklick auf Wert
//    Monat:=m; Jahr:=j;
//    mstart:=12*seVonJahr.Value+cbVonMonat.ItemIndex;
//    mdiff:=mstart-12*seBisJahr.Value-cbBisMonat.ItemIndex;
//    end
//  else begin
//    mstart:=0;
//    end;
  end;

end.
