(* Wetterdaten eingeben und bearbeiten

    2007-2024 J. Rathlev 24222 Schwentinental
      Web:  www.rathlev-home.de
      Mail: kontakt(a)rathlev-home.de

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   letzte nderung: Feb. 2024
   *)

unit Wetedit;

interface

uses
  Winapi.Windows, System.SysUtils, Winapi.Messages, System.Classes, Vcl.Graphics, Vcl.Controls,
  Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons, WetSub, WetListe;

const
  mrChg = $10;

type
  TWetEditor = class(TForm)
    MonatWahl: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    XTmin: TEdit;
    XT: TEdit;
    XTmax: TEdit;
    XDruck: TEdit;
    XRegen: TEdit;
    XKomm: TEdit;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    XTag: TEdit;
    MonatZur: TSpeedButton;
    QuartalZur: TSpeedButton;
    MonatVor: TSpeedButton;
    QuartalVor: TSpeedButton;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    Label7: TLabel;
    Label8: TLabel;
    LetzterTag: TSpeedButton;
    Januar: TSpeedButton;
    Dezember: TSpeedButton;
    ErsterTag: TSpeedButton;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    TabBtn: TBitBtn;
    cbSchnee: TCheckBox;
    procedure InitEditor(Sender: TObject);
    procedure EndeEdit(Sender: TObject);
    procedure AbbruchEdit(Sender: TObject);
    procedure MonInc(Sender: TObject);
    procedure NeuerMonat(Sender: TObject);
    procedure MonDec(Sender: TObject);
    procedure Monat3Zur(Sender: TObject);
    procedure Monat3Vor(Sender: TObject);
    procedure NeuerTag(Sender: TObject);
    procedure TagVor(Sender: TObject);
    procedure TagZur(Sender: TObject);
    procedure Tag5Zur(Sender: TObject);
    procedure Tag5Vor(Sender: TObject);
    procedure XTminExit(Sender: TObject);
    procedure XKommExit(Sender: TObject);
    procedure XTExit(Sender: TObject);
    procedure XTmaxExit(Sender: TObject);
    procedure XDruckExit(Sender: TObject);
    procedure XRegenExit(Sender: TObject);
    procedure XTMinKey(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure XTKey(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure XTMaxKey(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure XDruckKey(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure XRegenKey(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure XKommKey(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure XTagKey(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ErsterTagClick(Sender: TObject);
    procedure LetzterTagClick(Sender: TObject);
    procedure JanuarClick(Sender: TObject);
    procedure DezemberClick(Sender: TObject);
    procedure ListBtnClick(Sender: TObject);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbSchneeKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbSchneeClick(Sender: TObject);
  private
    { Private-Deklarationen }
    EdTag    : word;
    XKommRet : boolean;
    procedure ShowData;

  public
    { Public-Deklarationen }
    EdMonat,EdJahr : word;
    ArbeitsBuffer  : TDatBufTyp;
    DataChange     : boolean;
    function StartEdit (var LTag,Monat : word; Jahr : word) : boolean;

  end;

var
  WetEditor: TWetEditor;

(* Fehlermeldung *)
procedure Eingabefehler;

implementation

{$R *.DFM}

uses NumberUtils, StringUtils;

{-----------------------------------------------------------------------}
(* Fehlermeldung *)
procedure Eingabefehler;
begin
  ShowMessage ('Eingabefehler!');
  end;

{-----------------------------------------------------------------------}
procedure TWetEditor.FormCreate(Sender: TObject);
begin
  EdTag:=1;
  end;

procedure TWetEditor.ShowData;
var
  n : integer;
begin
  if EdTag<1 then EdTag:=1;
  n:=MonatsTage(EdMonat,EdJahr);
  if EdTag>n then EdTag:=n;
  Xtag.Text:=IntToStr(EdTag);
  with MonatWahl do begin
    ItemIndex:=pred(EdMonat);
    Text:=Items[ItemIndex];
    end;
  with ArbeitsBuffer[EdMonat,EdTag] do begin
    if Valid (TMin,TNoVal) then XTMin.Text:=FloatToStrX(TMin,5,1)
    else XTMin.Text:='';
    if Valid (T,TNoVal) then XT.Text:=FloatToStrX(T,5,1)
    else XT.Text:='';
    if Valid (TMax,TNoVal) then XTMax.Text:=FloatToStrX(TMax,5,1)
    else XTMax.Text:='';
    if Valid (Regen,RNoVal) then XRegen.Text:=FloatToStrX(Regen,5,1)
    else XRegen.Text:='';
    cbSchnee.Checked:=Schnee;
    if Valid (Druck,DNoVal) then XDruck.Text:=FloatToStrX(Druck,5,0)
    else XDruck.Text:='';
    XKomm.Text:=Komm;
  end;
end;

procedure TWetEditor.InitEditor(Sender: TObject);
begin
  Caption:='Daten fr '+IntToStr(EdJahr);
  ShowData; DataChange:=false; XTMin.SetFocus;
end;

procedure TWetEditor.EndeEdit(Sender: TObject);
var
  rc : integer;
begin
  if DataChange then rc:=mrOk+mrChg else rc:=mrOk;
  ModalResult:=rc;
end;

procedure TWetEditor.AbbruchEdit(Sender: TObject);
var
  rc : integer;
begin
  if DataChange then rc:=mrCancel+mrChg else rc:=mrCancel;
  ModalResult:=rc;
end;

procedure TWetEditor.NeuerMonat(Sender: TObject);
begin
  EdMonat:=succ(MonatWahl.ItemIndex);
  ShowData;
  XTMin.SetFocus;
end;

procedure TWetEditor.MonInc(Sender: TObject);
begin
  if EdMonat<12 then inc(EdMonat);
  ShowData;
  XTMin.SetFocus;
end;

procedure TWetEditor.MonDec(Sender: TObject);
begin
  if EdMonat>1 then dec(EdMonat);
  ShowData;
  XTMin.SetFocus;
end;

procedure TWetEditor.Monat3Zur(Sender: TObject);
begin
  if EdMonat>3 then dec(EdMonat,3) else EdMonat:=1;
  ShowData;
  XTMin.SetFocus;
end;

procedure TWetEditor.Monat3Vor(Sender: TObject);
begin
  if EdMonat<10 then inc(EdMonat,3) else EdMonat:=12;;
  ShowData;
  XTMin.SetFocus;
end;

procedure TWetEditor.JanuarClick(Sender: TObject);
begin
  EdMonat:=1;
  ShowData;
  XTMin.SetFocus;
end;

procedure TWetEditor.DezemberClick(Sender: TObject);
begin
  EdMonat:=12;
  ShowData;
  XTMin.SetFocus;
end;

procedure TWetEditor.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ssCtrl in Shift then begin
    if Key=VK_RIGHT then TagVor(Sender)
    else if Key=VK_LEFT then TagZur(Sender);
    Key:=0;
    end
  else begin
    if Key=VK_NEXT then MonInc(Sender)
    else if Key=VK_PRIOR then MonDec(Sender);
    end;
end;

procedure TWetEditor.NeuerTag(Sender: TObject);
var
  j,ic : integer;
begin
  val (XTag.Text,j,ic);
  if ic=0 then begin
    EdTag:=j; ShowData; XTMin.SetFocus;
    end
  else begin
    Eingabefehler; XTag.SetFocus;
    end;
end;

procedure TWetEditor.TagVor(Sender: TObject);
begin
  if ActiveControl is TEdit then (ActiveControl as TEdit).OnExit(ActiveControl);
  if EdTag<MonatsTage(EdMonat,EdJahr) then inc(EdTag)
  else if EdMonat<12 then begin
    inc(EdMonat); EdTag:=1;
    end;
  ShowData;
  if ActiveControl is TEdit then (ActiveControl as TEdit).SelectAll;
//  XTMin.SetFocus;
end;

procedure TWetEditor.TagZur(Sender: TObject);
begin
  if ActiveControl is TEdit then (ActiveControl as TEdit).OnExit(ActiveControl);
  if EdTag>1 then dec(EdTag)
  else if EdMonat>1 then begin
    dec(EdMonat); EdTag:=MonatsTage(EdMonat,EdJahr);
    end;
  ShowData;
  if ActiveControl is TEdit then (ActiveControl as TEdit).SelectAll;
//  XTMin.SetFocus;
end;

procedure TWetEditor.Tag5Zur(Sender: TObject);
begin
  if EdTag>5 then dec(EdTag,5)
  else if EdMonat>1 then begin
      dec(EdMonat); EdTag:=MonatsTage(EdMonat,EdJahr);
      end
    else EdTag:=1;
  ShowData;
  XTMin.SetFocus;
end;

procedure TWetEditor.Tag5Vor(Sender: TObject);
begin
  if EdTag<MonatsTage(EdMonat,EdJahr)-4 then inc(EdTag,5)
  else if EdMonat<12 then begin
      inc(EdMonat); EdTag:=1;
      end
    else EdTag:=31;
  ShowData;
  XTMin.SetFocus;
end;

procedure TWetEditor.ErsterTagClick(Sender: TObject);
begin
  EdTag:=1;
  ShowData;
  XTMin.SetFocus;
end;

procedure TWetEditor.LetzterTagClick(Sender: TObject);
begin
  EdTag:=MonatsTage(EdMonat,EdJahr);
  ShowData;
  XTMin.SetFocus;
end;

procedure TWetEditor.XTminExit(Sender: TObject);
var
  x : double;
  i : integer;
begin
  with XTMin do if Modified then begin
    DataChange:=true;
    if length(RemSp(Text))=0 then begin
      ArbeitsBuffer[EdMonat,EdTag].TMin:=TNoVal;
      end
    else begin
      if TryStrToFloat(Text,x) then begin
        ArbeitsBuffer[EdMonat,EdTag].TMin:=x;
        end
      else begin
        EingabeFehler; XTMin.SetFocus;
        end;
      end;
    end;
  ShowData;
end;

procedure TWetEditor.XTExit(Sender: TObject);
var
  x : double;
  i : integer;
begin
  with XT do if Modified then begin
    DataChange:=true;
    if length(RemSp(Text))=0 then begin
      ArbeitsBuffer[EdMonat,EdTag].T:=TNoVal;
      end
    else begin
      if TryStrToFloat(Text,x) then begin
        ArbeitsBuffer[EdMonat,EdTag].T:=x;
        end
      else begin
        EingabeFehler; XT.SetFocus;
        end;
      end;
    end;
  ShowData;
end;

procedure TWetEditor.XTmaxExit(Sender: TObject);
var
  x : double;
  i : integer;
begin
  with XTmax do if Modified then begin
    DataChange:=true;
    if length(RemSp(Text))=0 then begin
      ArbeitsBuffer[EdMonat,EdTag].TMax:=TNoVal;
      end
    else begin
      if TryStrToFloat(Text,x) then begin
        ArbeitsBuffer[EdMonat,EdTag].TMax:=x;
        end
      else begin
        EingabeFehler; XTmax.SetFocus;
        end;
      end;
    end;
  ShowData;
  end;

procedure TWetEditor.XDruckExit(Sender: TObject);
var
  x : double;
  i : integer;
begin
  with XDruck do if Modified then begin
    DataChange:=true;
    if length(RemSp(Text))=0 then begin
      ArbeitsBuffer[EdMonat,EdTag].Druck:=DNoVal;
      end
    else begin
      if TryStrToFloat(Text,x) then begin
        ArbeitsBuffer[EdMonat,EdTag].Druck:=x;
        end
      else begin
        EingabeFehler; XDruck.SetFocus;
        end;
      end;
    end;
  ShowData;
  end;

procedure TWetEditor.XRegenExit(Sender: TObject);
var
  x : double;
  i : integer;
begin
  with XRegen do if Modified then begin
    DataChange:=true;
    if length(RemSp(Text))=0 then begin
      ArbeitsBuffer[EdMonat,EdTag].Regen:=0.0;
      end
    else begin
      if TryStrToFloat(Text,x) then begin
        ArbeitsBuffer[EdMonat,EdTag].Regen:=x;
        end
      else begin
        EingabeFehler; XRegen.SetFocus;
        end;
      end;
    end
  else if length(RemSp(Text))=0 then begin
      ArbeitsBuffer[EdMonat,EdTag].Regen:=0.0;
      DataChange:=true;
      end;
  ShowData;
  end;

procedure TWetEditor.cbSchneeClick(Sender: TObject);
begin
  if cbSchnee.Checked<>ArbeitsBuffer[EdMonat,EdTag].Schnee then begin
    ArbeitsBuffer[EdMonat,EdTag].Schnee:=cbSchnee.Checked;
    DataChange:=true;
    end;
  end;

procedure TWetEditor.XKommExit(Sender: TObject);
begin
  with XKomm do if Modified then begin
    DataChange:=true;
    ArbeitsBuffer[EdMonat,EdTag].Komm:=Text;
    end;
  if XKommRet then begin
    XKommRet:=false;
    if EdTag<MonatsTage(EdMonat,EdJahr) then begin
      inc (EdTag);
      XTMin.SetFocus;
      end
    else if EdMonat<12 then begin
      EdTag:=1; inc(EdMonat);
      XTMin.SetFocus;
      end
    else OkBtn.SetFocus;
    end;
  ShowData;
  end;

procedure TWetEditor.XTMinKey(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then XT.SetFocus;
  end;

procedure TWetEditor.XTKey(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then XTMax.SetFocus;
end;

procedure TWetEditor.XTMaxKey(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then XDruck.SetFocus;
end;

procedure TWetEditor.XDruckKey(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then XRegen.SetFocus;
end;

procedure TWetEditor.XRegenKey(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then cbSchnee.SetFocus;
end;

procedure TWetEditor.cbSchneeKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then XKomm.SetFocus;
end;

procedure TWetEditor.XKommKey(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  XKommRet:=(Key=VK_RETURN);
  if XKommRet then XTMin.SetFocus;
end;

procedure TWetEditor.XTagKey(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then XTMin.SetFocus;
end;

procedure TWetEditor.ListBtnClick(Sender: TObject);
var
  f   : word;
  ed  : boolean;
begin
  f:=0;
  Liste.Caption:='Wetterdaten fr '+MonatWahl.Items[EDMonat-1]+'/'+IntToStr(EdJahr);
  Liste.ShowList (Arbeitsbuffer,EdMonat,EDJahr,EdTag,f,ed);
  XTMin.SetFocus;
  ShowData;
  end;

procedure TWetEditor.EditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#$D then Key:=#0;
end;

{-----------------------------------------------------------------------}
(* Wetterdaten bearbeiten/eingeben *)
function TWetEditor.StartEdit (var LTag,Monat : word;
                               Jahr : word) : boolean;
var
  rc : integer;
  ok : boolean;
begin
  EdMonat:=Monat; EdJahr:=Jahr; EdTag:=LTag; XKommRet:=false;
  rc:=ShowModal;
  ok:=(rc=(mrChg+mrOK)) or
      ((rc=(mrChg+mrCancel)) and
       (Application.MessageBox (
        'Es wurden Daten gendert. Sollen sie gespeichert werden?',
        'Warnung',MB_YESNO+MB_ICONQUESTION+MB_DEFBUTTON2)
        = IDYES));
  if ok or (rc and $F=mrOK) then begin
    Ltag:=EdTag; Monat:=EdMonat;
    end;
  StartEdit:=ok;
  end;

end.
