(* Monatsdaten in Listenform anzeigen

    2007-2024 J. Rathlev 24222 Schwentinental
      Web:  www.rathlev-home.de
      Mail: kontakt(a)rathlev-home.de

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   letzte nderung: Feb. 2024
   *)

unit Wetliste;

interface

uses
  Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Controls,
  Vcl.Forms, Vcl.Dialogs, Vcl.Grids, Vcl.StdCtrls, Vcl.Buttons, WetSub, Vcl.ExtCtrls;

type
  TListe = class(TForm)
    Panel2: TPanel;
    BitBtn1: TBitBtn;
    StringGrid: TStringGrid;
    procedure EndeClick(Sender: TObject);
    procedure GridClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen }
    NeuTag,NeuFeld : word;
    DKlick : boolean;
  public
    { Public-Deklarationen }
    procedure ShowList (const Buffer : TDatBufTyp;
                        Monat,Jahr   : word;
                        var Tag,Feld : word;
                        var EditData : boolean);
  end;

var
  Liste: TListe;

implementation

{$R *.DFM}

uses NumberUtils;

procedure TListe.EndeClick(Sender: TObject);
begin
  ModalResult:=mrOK; DKlick:=true;
  end;

procedure TListe.ShowList (const Buffer : TDatBufTyp;
                           Monat,Jahr   : word;
                           var Tag,Feld : word;
                           var EditData : boolean);
var
  i : integer;
begin
  NeuTag:=Tag; Feld:=NeuFeld; DKlick:=false;
  with StringGrid do begin
    Cells[0,0]:='Tag';
    Cells[1,0]:='T(min)/C'; Cells[2,0]:='T/C'; Cells[3,0]:='T(max)/C';
    Cells[4,0]:='p/hPa'; Cells[5,0]:='N/mm'; Cells[6,0]:=''; Cells[7,0]:='Komm.';
    RowCount:=MonatsTage(Monat,Jahr)+1;
    for i:=1 to RowCount-1 do with Buffer[Monat,i] do begin
      Cells[0,i]:=IntToStr(i);
      if Valid (TMin,TNoVal) then Cells[1,i]:=FloatToStrX(TMin,5,1)
      else Cells[1,i]:='';
      if Valid (T,TNoVal) then Cells[2,i]:=FloatToStrX(T,5,1)
      else Cells[2,i]:='';
      if Valid (TMax,TNoVal) then Cells[3,i]:=FloatToStrX(TMax,5,1)
      else Cells[3,i]:='';
      if Valid (Druck,DNoVal) then Cells[4,i]:=FloatToStrX(Druck,5,0)
      else Cells[4,i]:='';
      if Valid (Regen,RNoVal) then Cells[5,i]:=FloatToStrX(Regen,5,1)
      else Cells[5,i]:='';
      if Schnee then Cells[6,i]:='S' else Cells[6,i]:='';
      Cells[7,i]:=Komm;
      end;
    Row:=NeuTag; Col:=NeuFeld;
    end;
  Showmodal;
  Tag:=NeuTag; Feld:=NeuFeld; EditData:=DKlick;
  end;

procedure TListe.FormShow(Sender: TObject);
begin
  with StringGrid do begin
    ColWidths[0]:=40; ColWidths[1]:=60; ColWidths[2]:=60;
    ColWidths[3]:=60; ColWidths[4]:=60; ColWidths[5]:=60;
    ColWidths[6]:=20; ColWidths[7]:=Width-360;
    end;
  end;

procedure TListe.GridClick(Sender: TObject);
begin
  with StringGrid do begin
    NeuTag:=Row; NeuFeld:=Col;
    end;
  end;

end.
